function game_startup()

	game = {}
	game.selected_spell = 1
	graphics_startup()
	gui_startup()
	
	ents.Create("player", 400, 400)
	game_startup_done = true
	print("game startup done")
end

function graphics_startup()
	textures = {}
	textures.player = love.graphics.newImage("textures/player.png")
	textures.enemy = love.graphics.newImage("textures/enemy.png")
	textures.enemy_frozen = love.graphics.newImage("textures/enemy_frozen.png")
	textures.enemy_stunned = love.graphics.newImage("textures/enemy_stunned.png")
	
	textures.fireball = love.graphics.newImage("textures/fireball.png")
	textures.fireball_exp = love.graphics.newImage("textures/fireball_exp.png")
	
	textures.icenova = love.graphics.newImage("textures/icenova.png")
	textures.icenova_circle = love.graphics.newImage("textures/icenova_circle.png")
	textures.icenova_explosion = love.graphics.newImage("textures/icenova_explosion.png")
end

function get_world_curser_pos(t)
	local center_screen_x = screen_x/2
	local center_screen_y = screen_y/2
	local object_width = t.image:getWidth()
	local object_height = t.image:getHeight()
	local camera_offset_x = t.x - (center_screen_x * camera_scale) 
	local camera_offset_y = t.y - (center_screen_y * camera_scale) 
	local mouse_pos_x = (love.mouse.getX() * camera_scale) + camera_offset_x
	local mouse_pos_y = (love.mouse.getY() * camera_scale) + camera_offset_y

	return mouse_pos_x, mouse_pos_y
end

function camera_center_update(t)
	local center_screen_x = screen_x/2
	local center_screen_y = screen_y/2
	local object_width = t.image:getWidth()/2
	local object_height = t.image:getHeight()/2
	camera.x = t.x - (center_screen_x * camera_scale)
	camera.y = t.y - (center_screen_y * camera_scale)
end

function repel_object(source, target, hit_force)
	
	local difference_x = target.x - source.x
	local difference_y = target.y - source.y
	local hit_x_ratio = 0
	local hit_y_ratio = 0
	
	if difference_x >= 0 then --calculate initial initial force to the fireballds
		if difference_y >= 0 then 
			hit_x_ratio =  difference_x / (difference_x + difference_y)
			hit_y_ratio =  difference_y / (difference_x + difference_y)
		elseif difference_y < 0 then
			hit_x_ratio =  difference_x / (difference_x + (-difference_y))
			hit_y_ratio =  -1*((-difference_y) / (difference_x + (-difference_y)))
		end
		
	elseif difference_x < 0 then
		if difference_y >= 0 then
			hit_x_ratio =  -1*((-difference_x) / ((-difference_x) + difference_y))
			hit_y_ratio =  difference_y / ((-difference_x) + difference_y)
		elseif difference_y < 0 then
			hit_x_ratio =  -1*((-difference_x) / ((-difference_x) + (-difference_y)))
			hit_y_ratio =  -1*((-difference_y) / ((-difference_x) + (-difference_y)))
		end
	end
	target.body:applyForce(	hit_x_ratio * hit_force, 
							hit_y_ratio * hit_force)
							
	return hit_x_ratio, hit_y_ratio
end

function camera_zoom_update()
	local zoom_speed = 0.001
	local minscale = default_cam_scale
	local maxscale = default_cam_scale*1.4
	
	if love.keyboard.isDown("[") then
		camera_scale = camera_scale - zoom_speed
		if camera_scale < minscale then
			camera_scale = minscale
		end
	elseif love.keyboard.isDown("]") then
		camera_scale = camera_scale + zoom_speed
		if camera_scale > maxscale then
			camera_scale = maxscale
		end
	end
	
	camera:setScale(camera_scale, camera_scale)
end

function cleanup_rubbish()
	for i, ent in pairs(ents.objects) do
		if ent.rubbish == true then
			print (ent.type)
			ent.body:destroy()
			ent = nil
		end
	end
end

function gui_startup()  --split into game and editor
	gui = {}
	gui.cursors = {}
	gui.cursors.editor 	= love.graphics.newImage("textures/editor_cursor.png")
	gui.cursors.fire 	= love.graphics.newImage("textures/fire_cursor.png")
	gui.cursors.ice 	= love.graphics.newImage("textures/ice_cursor.png")
	
	gui.spell_icons = {}
	gui.spell_icons.fire = love.graphics.newImage("textures/gui_fire.png")
	gui.spell_icons.ice = love.graphics.newImage("textures/gui_ice.png")
	gui.spell_icons.select = love.graphics.newImage("textures/gui_selection.png")
	
	
	love.mouse.setVisible(false)
	cursor_visible = true
	gui_startup_done = true
end

function draw_cursor()		--FIX THIS UP
	local cursor_image = ""
	if cursor_visible == true then 
		if game_startup_done then
			for i, ent in pairs(ents.objects) do
				if ent.type == "player" then
					if ent.selected_spell == 1 then
						cursor_image = gui.cursors.fire
					elseif ent.selected_spell == 2 then
						cursor_image = gui.cursors.ice
					end
				end
			end
		end
		
		if editor_startup_done == true then
			cursor_image = gui.cursors.editor
		end
		
		love.graphics.draw(cursor_image,love.mouse.getX() - cursor_image:getWidth()/2, 
										love.mouse.getY() - cursor_image:getHeight()/2)
	
	end
end

function draw_gui()
	if gui_startup_done == true then
		local gui_offset_X	= 0
		local gui_offset_Y	= 550
		
		love.graphics.setColor( 255, 255, 255, 255)
		for i, ent in pairs(ents.objects) do
			if ent.type == "player" then
				if ent.fire_delayed == true then
					love.graphics.setColor( 50, 50, 50, 255)
				end
				
				love.graphics.draw(gui.spell_icons.fire, gui_offset_X, gui_offset_Y)
				love.graphics.setColor( 255, 255, 255, 255)
				
				if ent.ice_delayed == true then
					love.graphics.setColor( 50, 50, 50, 255)
				end
				
				love.graphics.draw(gui.spell_icons.ice, gui_offset_X+100, gui_offset_Y)
				love.graphics.setColor( 255, 255, 255, 255)
				
				love.graphics.draw(gui.spell_icons.select, gui_offset_X + (ent.selected_spell* gui.spell_icons.ice:getWidth()) -  gui.spell_icons.ice:getWidth(), gui_offset_Y)
				
				if ent.fire_release_time then
						love.graphics.print(ent.fire_delay_max - (love.timer.getTime() - ent.fire_release_time), 
											(gui_offset_X + (gui.spell_icons.fire:getWidth()/2)), 
											(gui_offset_Y + gui.spell_icons.fire:getHeight()/2))
				end
				
				if ent.ice_release_time then
						love.graphics.print(ent.ice_delay_max - (love.timer.getTime() - ent.ice_release_time), 
											(gui_offset_X + gui.spell_icons.ice:getWidth() + (gui.spell_icons.ice:getWidth()/2)), 
											gui_offset_Y + gui.spell_icons.ice:getHeight()/2)
				end
			end
		end
	end
end