function map_startup()
	--generate map
	local current_map = "map1.lua"
	local map_path = "states/game/maps/"
	load_map(current_map, map_path)
	
	map_variable_setup()
	create_map_visuals(true)
	create_map_physics()
	map_startup_done = true
end

function map_variable_setup()
	-- map variables
	map_offset_x = 0  		--map offset from 0
	map_offset_y = 0  		--GLOBAL
	map_display_w = 18  	--how much of the map is displayed
	map_display_h = 18		--GLOBAL
	tile_w = 64  			--tile dimensions
	tile_h = 64 			--GLOBAL
	vis_center_x = 400 		--GLOBAL
	vis_center_y = 400 		--GLOBAL
	
	
	objects = {}  			--GLOBAL
	objects.phys_map = {}  	--GLOBAL
	
	tiles = {}  			--GLOBAL
	for y = 1, map_h do			
		local new_table = {}
		for x = 1, map_w do
			new_table.x = ""
		end
		table.insert(tiles, new_table)
	end	
	
end

function create_new_map(new_map_size_x, new_map_size_y)
	local temp_map = {}
	
	map = {}
	
	for y = 1, new_map_size_y do
		for x = 1, new_map_size_x do
			temp_map[x] = 1
		end

		table.insert(map, temp_map)
		temp_map = {}
	end
	
	map_h = new_map_size_y
	map_w = new_map_size_x
end

function load_map(current_map, map_path)
	
	local file = love.filesystem.newFile(map_path..current_map)
	local temp_string_map = {}
	local new_map = {}
	
	map = {}  --GLOBAL

	for line in love.filesystem.lines(map_path..current_map) do
		table.insert(temp_string_map, line)
	end
	
	map_h = #temp_string_map
	map_w = string.len(temp_string_map[1])
	
	for y = 1, map_h do
		new_map = {}
		tempstring = string.sub(temp_string_map[y], 1, map_w)
		for x = 1, map_w do
			new_map[x] = tonumber(string.sub(tempstring, x, x))
		end
		
		table.insert(map, new_map)
	end	
end

function save_map()  --BROKEN
	local i = 1
	local fileexists = true
	local newfilenum
	
	print("saving...")
	
	while fileexists == true do
		i = i + 1
		fileexists = love.filesystem.exists( "map"..i..".lua" ) --returns the existance of a file
		if fileexists == false then
			newfilenum = i
		end
	end
	
	local file = love.filesystem.newFile("map"..newfilenum..".lua")
	file:open('a')
	
	local temp_string = ""

	for x=1, map_w do 
		
		for y=1, map_h do 
			temp_string = temp_string..map[y][x]
		end
		
		temp_string = temp_string.."\n"

	end

	file:write(temp_string)
	file:close()
	print("...saved to map"..newfilenum..".lua")	
end

function create_map_visuals(full, x, y) --is this a full map rebuild?
	
	if full == true then
		for y=1, map_h do --draw map_display_h tiles in y
			for x=1, map_w do --draw map_display_w tiles in w  
				calculate_wall_tile(x, y)
			end
		end	
	end
	
	if full == false then
		for i = y-1, y+1 do
			for j = x-1, x+1 do
				calculate_wall_tile(j, i)
			end
		end
	end
end

function calculate_wall_tile(x, y)
	local tilenumber = ""
	local tilenum = ""
	local tilename = "tile_error.png"
	
	if map[y][x] == 1 then --if the map piece is a wall
		tilenumber = ""
		
		for y1 = -1, 1 do
			for x1 = -1, 1 do
			
				if 	x1 == 0 and y1 == 0 then
				else
					tilenum = 1
				
						if y + y1 > 0 and y + y1 <= map_h then
							if x + x1 > 0 and x + x1 <= map_w then
							tilenum = map[y+y1][x+x1]
							
							
							if x+x1 == x+1 then
								if y+y1 == y-1 or y+y1 == y+1 then
									if	x+1 <= map_w then  
										if map[y][x+1] == 0 then
											tilenum = 0
										end
									end
								end
							end
							
							if x+x1 == x-1 then
								if y+y1 == y-1 or y+y1 == y+1 then
									if 	x-1 > 0 then  
										if map[y][x-1] == 0 then
											tilenum = 0
										end
									end
								end
							end 
							
							if y+y1 == y+1 then
								if x+x1 == x-1 or x+x1 == x+1 then
									if 	y+1 <= map_h then  
										if map[y+1][x] == 0 then
											tilenum = 0
										end
									end
								end
							end
								
							if y+y1 == y-1 then
								if x+x1 == x-1 or x+x1 == x+1 then
									if y-1 > 0 then
										if map[y-1][x] == 0 then
											tilenum = 0
										end
									end
								end
							end
							
						end
					end
					
					tilenumber = tilenumber..tilenum
					
				end
			end
		end
	else
		tilenumber = "_floor"
	end

	if love.filesystem.exists("textures/tile"..tilenumber..".png") then
		tilename = "textures/tile"..tilenumber..".png"
	else
		tilename = "textures/tile_error.png"
	end
	
	tiles[y][x] = love.graphics.newImage(tilename)
end


function create_map_physics()
	objects.worldmap = {}
	for x=1, map_w do --draw map_display_w tiles in w  
		for y=1, map_h do --draw map_display_h tiles in y
		
			if map[y][x] == 1 then
				
				local new_table = {}
				
				new_table.image = love.graphics.newImage("textures/tile_error.png")
				
				local tile_width = new_table.image:getWidth()
				local tile_height = new_table.image:getHeight()
				
				new_table.body = love.physics.newBody(world, (x*tile_width) + map_offset_x + tile_width/2, (y*tile_height) + map_offset_y + tile_height/2, "static") --place the body in the center of the world and make it dynamic, so it can move around
				tilex, tiley = new_table.body:getPosition()
				
				new_table.shape = love.physics.newRectangleShape(tile_width, tile_height)				
				new_table.fixture = love.physics.newFixture(new_table.body, new_table.shape, 1)
				new_table.fixture:setUserData("wall") 
				table.insert(objects.worldmap, new_table)
				
			end
		end
	end
end

function draw_map()
	local center_x = 0
	local center_y = 0
	local tile_x = 0
	local tile_y = 0
	
	for i, ent in pairs(ents.objects) do
		if ent.type == "player" or ent.type == "editor_player" then
			center_x = ent.x
			center_y = ent.y
		end
	end
	
	while center_x > 0 do
		tile_x = tile_x + 1
		center_x = center_x - tile_w
	end
	
	while center_y > 0 do
		tile_y = tile_y + 1
		center_y = center_y - tile_h
	end
	
	for y = 1 + tile_y - (map_display_h/2), map_display_h + tile_y - (map_display_h/2) do --draw map_display_h tiles in y
		for x = 1 + tile_x - (map_display_w/2), map_display_w + tile_x - (map_display_w/2) do --draw map_display_w tiles in w 
			if x > 0 and x <= map_w then
				if y > 0 and y <= map_h then
					love.graphics.draw( 
							tiles[y][x],
							(x*tile_w)+map_offset_x,
							(y*tile_h)+map_offset_y)
				end
			end
			--love.graphics.print("x: "..x.." y: "..y , (x*tile_w)+map_offset_x, (y*tile_h)+map_offset_y)
		end
	end
end

function draw_map_overlay()

	love.graphics.setColor(255, 255, 255, 50)
	love.graphics.setLineWidth(1)

	for x=1, map_w+1 do                                                  
		love.graphics.line(x*tile_w, tile_h, x*tile_w,(map_h*tile_h)+tile_h)
	end
		
	for y=1, map_h+1 do 	
		love.graphics.line(tile_w, y*tile_h,(map_w*tile_w)+tile_w, y*tile_h)
	end
	
	love.graphics.setLineWidth(1)
	love.graphics.setColor(255, 255, 255, 255)
end
