local ent = ents.Derive("base")

function ent:load (x, y)
	self.image = textures.player
	self.w = self.image:getWidth()
	self.h = self.image:getHeight()
	
	self.body = love.physics.newBody(world, 0, 0, "dynamic")
	self.body:setLinearDamping(2)
	self.body:setMass(1)
	self.shape = love.physics.newCircleShape(self.w/2)
	self.fixture = love.physics.newFixture(self.body, self.shape, 1) -- Attach fixture to body and give it a density of 1.
	self.fixture:setUserData("player") 

	self.maxhealth = 100
	self.health = self.maxhealth
	
	self.selected_spell = 1 
	self.fire_release_time = 0
	self.fire_delayed = false
	self.fire_cast_timer = 0
	
end

function ent:setPos( x, y, a )
	self.x = x
	self.y = y
	self.a = a
	self.body:setPosition( x, y )
end

function ent:getPos()
	return self.x, self.y, self.a;
end

function ent:setSize(w, h)
	self.w = w
	self.h = h
end

function ent:getSize()
	return self.w, self.h;
end

function ent:update(dt)
	self:update_movement()

end


function ent:register_keys(key, state)
	
	if key == "w" then
		self.up = state
	end

	if key == "a" then
		self.left = state
	end

	if key == "s" then
		self.down = state
	end

	if key == "d" then
		self.right = state
	end
end

function ent:update_movement()
	local move_force = 800
	local move_diag_force = move_force*0.6
		
	self.world_mouse_x, self.world_mouse_y = get_world_curser_pos(self)
	camera_center_update(self)
	
	self.body:setAngle(math.getAngle(self.x, self.y, self.world_mouse_x, self.world_mouse_y) + math.pi * 0.5)
	
	self.x, self.y = self.body:getPosition()
	self.a = self.body:getAngle()
	
	if not self.spellcasting then
		self.body:setLinearDamping(2)
		if self.up then 	
			if self.y > tile_h then
				if self.left then 												--and left
					self.body:applyForce(-move_diag_force, -move_diag_force) 	--up left
				elseif self.right then 											--and right
					self.body:applyForce(move_diag_force, -move_diag_force)  	--up right	
				elseif self.down then											--and down
				else
					self.body:applyForce(0, -move_force)						--just up
				end
			end

		elseif self.down then 													--if down
			if self.y < map_h*tile_h then
				if self.left then 												--and left
					self.body:applyForce(-move_diag_force, move_diag_force)		--down left
				elseif self.right then 											--and right
					self.body:applyForce(move_diag_force, move_diag_force)		--down right
				else
					self.body:applyForce(0, move_force)							--just down
				end
			end
			
		elseif self.left then 													--if left
			if self.x > tile_w then
				if self.right then 												--and right
				else
					self.body:applyForce(-move_force, 0)						--just left
				end
			end
			
		elseif self.right then 													--if right
			if self.x < map_w * tile_w then
				self.body:applyForce(move_force, 0)
			end
		end	
	else
		self.body:setLinearDamping(5)
	end
end

function ent:draw()
end

function ent:Die()
end

return ent;