local ent = ents.Derive("base")

function ent:load( x, y )
	enemy_count = enemy_count + 1
	self.birth = love.timer.getTime()

	self.ang = 0
	self.maxhealth = 100
	self.health = self.maxhealth
	
	self.exp_hit = false
	self.stun_time_max = 0.5
	self.stun_time = 0
	self.stunned = false
	
	self.maxfrozen_time = 3
	self.frozen_time = 0
	self.frozen = false
	
	
	self.image = textures.enemy
	self.w = self.image:getWidth()
	self.h = self.image:getHeight()
	
	self.body = love.physics.newBody(world, 0, 0, "dynamic")
	self.body:setLinearDamping(2)
	self.body:setMass(1)
	self.shape = love.physics.newCircleShape(self.w/2)
	self.fixture = love.physics.newFixture(self.body, self.shape, 1) 
	self.fixture:setUserData("enemy"..enemy_count)
	self.x, self.y = self.body:getPosition()	
	
end

function ent:setPos( x, y, a )
	self.x = x
	self.y = y
	self.a = a
	self.body:setPosition( x, y )
end

function ent:getPos()
	return self.x, self.y, self.a;
end

function ent:setSize(w, h)
	self.w = w
	self.h = h
end

function ent:getSize()
	return self.w, self.h;
end

function ent:update(dt)
	self.x, self.y = self.body:getPosition()	
	self.a = self.body:getAngle()
	
	self:update_movement()
	
	if self.colliding then
		self:update_collision()
	end
	
	if self.frozen == true then
		self:update_frozen()
	end
	
	if self.exp_hit == true then
		self.exp_hit = false
		self.stunned = true
		self.stun_time_max = 0.5
		
	end
	
	if self.stunned == true then
		self:update_stunned()
	end
	
	if self.health <= 0 then
		self.health = 0
		self:die(self.id)
		return
	end
end

function ent:update_stunned()
	
	if self.stun_time == 0 then
		self.stun_time = love.timer.getTime()
		
	elseif self.stun_time > 0 then
		if (love.timer.getTime() - self.stun_time) > self.stun_time_max then
			self.stun_time = 0
			self.stunned = false
		end
	end	
end

function ent:update_frozen()
	
	if self.frozen_time == 0 then
		self.frozen_time = love.timer.getTime()
		
	elseif self.frozen_time > 0 then
		if (love.timer.getTime() - self.frozen_time) > self.maxfrozen_time then
			self.frozen_time = 0
			self.frozen = false
			self.image = textures.enemy
		end
	end
end

function ent:update_movement()
	
	local move_force = 500
	local enemy_x = 0 
	local enemy_y = 0
	local difference_x = 0
	local difference_y = 0
	local enemy_ratio_x = 0
	local enemy_ratio_y = 0
	
	local enemy_movement = false
	if enemy_movement == true then 
		if self.frozen == true then
			self.body:setLinearDamping(2000)
		elseif self.stunned == true then
		else
			self.body:setLinearDamping(2)
			self.x, self.y = self.body:getPosition()
			
			for i, ent in pairs(ents.objects) do
				if ent.type == "player" then
					difference_x = ent.x - self.x
					difference_y = ent.y - self.y
				end
			end

			if difference_x > 0 then --calculate initial initial force to the enemy
				if difference_y > 0 then 
					enemy_ratio_x =  difference_x / (difference_x + difference_y)
					enemy_ratio_y =  difference_y / (difference_x + difference_y)
				elseif difference_y < 0 then
					enemy_ratio_x =  difference_x / (difference_x + (-difference_y))
					enemy_ratio_y =  -1*((-difference_y) / (difference_x + (-difference_y)))
				end
				
			elseif difference_x < 0 then
				if difference_y > 0 then
					enemy_ratio_x =  -1*((-difference_x) / ((-difference_x) + difference_y))
					fireball_y_ratio =  difference_y / ((-difference_x) + difference_y)
				elseif difference_y < 0 then
					enemy_ratio_x =  -1*((-difference_x) / ((-difference_x) + (-difference_y)))
					enemy_ratio_y =  -1*((-difference_y) / ((-difference_x) + (-difference_y)))
				end
			end			
			
			self.body:applyForce(	enemy_ratio_x * move_force, 
									enemy_ratio_y * move_force)
			
		end
	end
end

function ent:update_collision()
	
	for i, ent in pairs(ents.objects) do
		if ent.fixture:getUserData() == self.colliding then
			if ent.colliding then
				print("-------------------------------------------------------------------------------")
				print (ent.fixture:getUserData(), ent.colliding)
			end
			
			if ent.type == "fireball" then
				self:damage (ent.dam)
			end
			
			if ent.type == "fireball_exp" then
				self:damage (ent.dam)
				
				if ent.cast_time > 0.5 then
					if self.health > 0 then
						local hit_force = 100000 * ent.cast_time
						
						if ent.cast_time == 1 then
							hit_force = 150000
						end
						
						repel_object(ent, self,  hit_force)
						self.exp_hit = true
					end
				end
			end
			
			if self.health > 0 then
				if ent.type == "icenova" then
					self.frozen = true
				end
			end			
		end
	end
end

function ent:damage (n)
	if n <= 0 then return end
	self.health = self.health - n
end

function ent:die()
	enemy_count = enemy_count - 1
	ents.destroy(self.id)
end

function ent:draw()

	love.graphics.draw( self.image, self.x, self.y, self.a, 1, 1, self.w/2, self.h/2)
	
	if self.frozen == true then
		love.graphics.draw( textures.enemy_frozen, self.x, self.y, self.a, 1, 1, self.w/2, self.h/2)
	end
	
	if self.stunned == true then
		love.graphics.draw( textures.enemy_stunned, self.x, self.y, self.a, 1, 1, self.w/2, self.h/2)
	end
	
	love.graphics.print(self.health, self.x, self.y)
end

return ent;