local ent = ents.Derive("base_player")

function ent:load (x, y)
	self.image = textures.player
	self.w = self.image:getWidth()
	self.h = self.image:getHeight()
	
	self.body = love.physics.newBody(world, 0, 0, "dynamic")
	self.body:setLinearDamping(2)
	self.body:setMass(1)
	self.shape = love.physics.newCircleShape(self.w/2)
	self.fixture = love.physics.newFixture(self.body, self.shape, 1) -- Attach fixture to body and give it a density of 1.
	self.fixture:setUserData("player") 

	self.maxhealth = 100
	self.health = self.maxhealth
	
	self.fire_cast = true
	
	self.selected_spell = 1 
	self.spellcasting = false
	
	--self.fire_release_time = 0
	self.fire_delayed = false
	self.fire_cast_timer = 0
	self.fire_delay_max = 0.5
	
	self.ice_target_scale = 0
	self.ice_delayed = false
	self.ice_delay_max = 5
	
end

function ent:setPos( x, y, a )
	self.x = x
	self.y = y
	self.a = a
	self.body:setPosition( x, y )
end

function ent:getPos()
	return self.x, self.y, self.a;
end

function ent:setSize(w, h)
	self.w = w
	self.h = h
end

function ent:getSize()
	return self.w, self.h;
end

function ent:update(dt)

	self.x, self.y = self.body:getPosition()	
	self.a = self.body:getAngle()
	self:update_movement()
	self:update_spells(dt)
	if self.colliding then
		self:update_collision()
	end
end

function ent:update_spells(dt)

		if self.spell_cast == 1 then 
			if self.fire_delayed == false then
				self:cast_fireball(dt)
			end
		elseif self.spell_cast == 2 then
			if self.ice_delayed == false then
				self:cast_icenova(dt)
			end
		end	
	
	if self.fire_release_time then
		if love.timer.getTime() - self.fire_release_time > self.fire_delay_max then
			self.fire_delayed = false
			self.fire_release_time = nil
		end
	end

	if self.ice_release_time then
		if love.timer.getTime() - self.ice_release_time > self.ice_delay_max then
			self.ice_delayed = false
			self.ice_release_time = nil
		end
	end	
end

function ent:reset_spells(spell_number)
	self.spell_cast = 0
	if spell_number == 1 then
		self.fire_cast_timer = 0
		self.fire_dam = 0		
		self.fire_exp = false
	elseif spell_number == 2 then
		self.ice_mouse_x = nil
		self.ice_mouse_y = nil
		self.ice_target_scale = 0
	end
end

function ent:cast_fireball(dt)
	------------------------------------------------------------------------
	-- do fireball caster and calculate for release
	------------------------------------------------------------------------
	local fireball_cast_time_max = 1
	local fire_delay_timer_max = 2
	local fire_dam_min = 10
	local fire_dam_add = 60
	local fire_cast_max = 1
	
	if self.spell_released == false and self.spellcasting == true then
		self.fire_cast_timer = self.fire_cast_timer + (dt/fire_cast_max)
		
		self.fire_dam = fire_dam_min + (fire_dam_add * self.fire_cast_timer)
		
		if self.fire_dam > (fire_dam_min + fire_dam_add) then
			self.fire_dam = fire_dam_min + fire_dam_add
		end
		
		if self.fire_cast_timer > 0.5 then
			self.fire_exp = true	--fireball will explode on hit
		end
		
		if self.fire_cast_timer > 1 then
			self.fire_cast_timer = 1
		end		
	
	elseif self.spell_released == true then
		self.fire_delayed = true
		ents.Create("fireball", self.x, self.y, true, self.fire_dam, self.fire_exp, self.world_mouse_x, self.world_mouse_y, self.fire_cast_timer)
		self.fire_release_time = love.timer.getTime()
		self:reset_spells(1)
	end
end

function ent:cast_icenova(dt)
	local ice_target_scale_time = 1
	
	if self.spell_released == false and self.spellcasting == true then
		cursor_visible = false
			
		if not self.ice_mouse_x then
			self.ice_mouse_x, self.ice_mouse_y = get_world_curser_pos(self)
		end
		
		self.ice_target_scale = self.ice_target_scale + (dt / ice_target_scale_time)
		
		if self.ice_target_scale > 1 then 
			self.ice_target_scale = 1
		end		
	
	elseif self.spell_released == true then

		self.ice_delayed = true
		cursor_visible = true
		ents.Create("icenova", self.ice_mouse_x, self.ice_mouse_y, true, self.ice_target_scale)
		self.ice_release_time = love.timer.getTime()
		self:reset_spells(2)
	end
end

function ent:start_spell(x, y)
	self.spellcasting = true
	self.spell_released = false
	
	if self.selected_spell == 1 then
		self.spell_cast = 1

	elseif self.selected_spell == 2 then
		self.spell_cast = 2

	end

end

function ent:release_spell(x, y)
	self.spellcasting = false
	if self.selected_spell == 1 and self.fire_delayed == false then

		self.spell_released = true
	elseif self.selected_spell == 2 and self.ice_delayed == false then	

		self.spell_released = true
	end
end

function ent:select_spell(key)
	self.reset_spell = self.selected_spell --reset the spell we changed from
	self.selected_spell = tonumber(key)	-- 1 fireball, 2 nova
	
	if self.reset_spell ~= self.selected_spell then 
		self:reset_spells(self.reset_spell)
	end
	
	self.spell_released = false
	self.spellcasting = false
end

function ent:register_keys(key, state)
	
	if key == "w" then
		self.up = state
	end

	if key == "a" then
		self.left = state
	end

	if key == "s" then
		self.down = state
	end

	if key == "d" then
		self.right = state
	end
end

function ent:update_movement()
	local move_force = 800
	local move_diag_force = move_force*0.6
		
	self.world_mouse_x, self.world_mouse_y = get_world_curser_pos(self)
	camera_center_update(self)
	
	self.body:setAngle(math.getAngle(self.x, self.y, self.world_mouse_x, self.world_mouse_y) + math.pi * 0.5)
	
	self.x, self.y = self.body:getPosition()
	self.a = self.body:getAngle()
	
	if self.spellcasting == false then
		self.body:setLinearDamping(2)
		if self.up then 	
			if self.y > tile_h then
				if self.left then 												--and left
					self.body:applyForce(-move_diag_force, -move_diag_force) 	--up left
				elseif self.right then 											--and right
					self.body:applyForce(move_diag_force, -move_diag_force)  	--up right	
				elseif self.down then											--and down
				else
					self.body:applyForce(0, -move_force)						--just up
				end
			end

		elseif self.down then 													--if down
			if self.y < map_h*tile_h then
				if self.left then 												--and left
					self.body:applyForce(-move_diag_force, move_diag_force)		--down left
				elseif self.right then 											--and right
					self.body:applyForce(move_diag_force, move_diag_force)		--down right
				else
					self.body:applyForce(0, move_force)							--just down
				end
			end
			
		elseif self.left then 													--if left
			if self.x > tile_w then
				if self.right then 												--and right
				else
					self.body:applyForce(-move_force, 0)						--just left
				end
			end
			
		elseif self.right then 													--if right
			if self.x < map_w * tile_w then
				self.body:applyForce(move_force, 0)
			end
		end	
	else
		self.body:setLinearDamping(5)
	end
end

function ent:update_collision()
	local hit_force = 200000
	
	for i, ent in pairs(ents.objects) do
		if ent.fixture:getUserData() == self.colliding then
			if ent.type == "enemy" then
				repel_object(self, ent, hit_force)
				self.health = self.health - 10
			end
		end
	end
end

function ent:draw()
	
	if self.spell_cast == 1 then
		if self.fire_delayed == false and self.selected_spell == 1 then	
			self:draw_fireball_cast()
		end
	elseif self.spell_cast == 2 then
		if self.ice_delayed == false and self.selected_spell == 2 then
			self:draw_icenova_cast()
		end
	end
	
	love.graphics.draw( self.image, self.x, self.y, self.a, 1, 1, self.w/2, self.h/2)

end

function ent:draw_icenova_cast()
	
	if self.ice_mouse_x then
	love.graphics.draw(	textures.icenova_circle, 
						self.ice_mouse_x - textures.icenova_circle:getWidth()/2, 
						self.ice_mouse_y - textures.icenova_circle:getHeight()/2) --draw the outer circle

	love.graphics.draw(	textures.icenova_explosion, 
						self.ice_mouse_x, self.ice_mouse_y, 
						0, self.ice_target_scale, self.ice_target_scale, 
						textures.icenova_explosion:getWidth()/2, 
						textures.icenova_explosion:getHeight()/2) --draw the inner circle
	end
end

function ent:draw_fireball_cast(cast_start, dt)
	local charge_radius = 0
	local overcharge = 0

	if self.fire_cast_timer <= 0.5 then
	
		charge_radius =	self.image:getWidth()/2 + 
						((self.image:getWidth()/2)*(self.fire_cast_timer*2))	
		
		love.graphics.setColor(255, 0, 0)
		
	elseif self.fire_cast_timer > 0.5 then
	
		charge_radius =	self.image:getWidth()
		
		love.graphics.setColor(255, ((self.fire_cast_timer-0.5)*2)*255, 
									((self.fire_cast_timer-0.5)*2)*255)
	end

	
	love.graphics.circle("fill", self.x, self.y, charge_radius)
	love.graphics.setColor(255, 255, 255)	
	
	
		
end

function ent:Die()
end

return ent;