function load()     
	require("states/game/code/menu")
	require("states/game/code/camera")
	require("states/game/code/game")
	require("states/game/code/map")
	require("states/game/code/collision")
	require("states/game/code/editor")
	require("states/game/entities")
	require("states/game/maps")
	logmessage = ""
	
	ents.Startup()
	
	love.physics.setMeter(32) 
	world = love.physics.newWorld(0, 0, true) 
		world:setCallbacks(beginContact, endContact, preSolve, postSolve)
		
	screen_x, screen_y, fullscreen, vsync, fsaa = love.graphics.getMode( )
	
	default_cam_scale = 1   --GLOBAL
	camera_scale = default_cam_scale --GLOBAL
	camera:setScale(camera_scale, camera_scale) --GLOBAL
	
	startup()
end

function startup()
	start_game = false
	in_front_end = true
	map_startup_done = false
	game_startup_done = false
	front_end_startup()
end

--------------------------------------------------------------------------------------------------------
-- update
--------------------------------------------------------------------------------------------------------

function love.update(dt)
	
	world:update(dt)
	
	if in_front_end == true then
		front_end()
	elseif in_front_end == false then
		if start_game == true then	
			game_startup()
			map_startup()
					
			enemy_count = 0
			fireball_count = 0
			fireball_exp_count = 0
			icenova_count = 0
					
			for i, j in pairs (ents.objects) do
				print (ents.objects[i].type, ents.objects[i].id)
			end
			start_game = false
		end
		
		if start_editor == true then
			editor_startup(true, newmap_x, newmap_y)
			start_editor = false
		end

		if game_startup_done == true then
			if enemy_count == 0 then
				ents.Create("enemy", 200, 200)
				ents.Create("enemy", 300, 200)
				ents.Create("enemy", 400, 200)
				ents.Create("enemy", 500, 200)
			end
		end
		
		if editor_startup_done == true then
			editor_main()
		end
		
		ents:update(dt)
		
		if game_startup_done == true then
			clear_collision()
		end
	end
end

--------------------------------------------------------------------------------------------------------
-- draw
--------------------------------------------------------------------------------------------------------

function love.draw()
	if in_front_end == true then
		draw_frontend()
	elseif in_front_end == false then
		if map_startup_done == true then
			camera:set()
				draw_map()
				
				camera_zoom_update()
				
				ents:drawBG()
				ents:draw()
			camera:unset()
		end
		
		if game_startup_done == true then 
			draw_gui()
			draw_cursor()
		end
		
		if editor_startup_done == true then
		
			camera:set()
				draw_map()
				draw_map_overlay()
				
				camera_zoom_update()

				ents:draw()
			camera:unset()
			
			draw_cursor()
		end
	end
	love.graphics.print(logmessage, 20, 200)
end

--------------------------------------------------------------------------------------------------------
--other functions
--------------------------------------------------------------------------------------------------------

function math.getAngle(x1,y1, x2,y2) return math.atan2(y2-y1, x2-x1) end

function love.focus(bool)
end

function love.keypressed( key, unicode )
	if key == "x" then
		if editor_startup_done then 
			save_map()
		end
	end
	
	if game_startup_done or editor_startup_done then
		for i, ent in pairs(ents.objects) do
			if ent.type == "player" or  ent.type == "editor_player" then
				if ent.type == "player" then
					if key == "1" or key == "2" then
						ent:select_spell( key )
					end
				end
				
				if key == "w" or key == "a" or key == "s" or key == "d" then
					ent:register_keys(key, true)
				end
			end
		end
	end	
end

function love.keyreleased( key, unicode )
	if game_startup_done or editor_startup_done then
		for i, ent in pairs(ents.objects) do
			if ent.type == "player" or  ent.type == "editor_player" then
				if key == "w" or key == "a" or key == "s" or key == "d" then
					ent:register_keys(key, false)
				end
			end
		end
	end
end

function love.mousepressed( x, y, button )
	
	if game_startup_done then
		for i, ent in pairs(ents.objects) do
			if ent.type == "player" then
				if button == "l" then
					ent:start_spell( x, y )
				end
			end
		end
	end
	
	if editor_startup_done then 
		if button == "l" then
			check_mouse_clicks(editor_get_mouse())
		end
	end
end

function love.mousereleased( x, y, button )
	if button == "l" then
		if game_startup_done then
			for i, ent in pairs(ents.objects) do
				if ent.type == "player" then
					ent:release_spell( x, y )
				end
			end
		end
	end
end

function love.quit()
end